package com.jh.common.security;

import org.springframework.context.annotation.Bean;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.web.servlet.config.annotation.ResourceHandlerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurerAdapter;

/**
 * The Class ReactApplicationSecurityContext.
 */
public class ReactApplicationSecurityContext extends WebSecurityConfigurerAdapter {
    
    /**
     * @param http
     * @throws Exception
     * @see org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter#configure(org.springframework.security.config.annotation.web.builders.HttpSecurity)
     */
    @Override
    protected void configure(HttpSecurity http) throws Exception {
        http
	        .authorizeRequests()
	        	.antMatchers("/api/auth/signup").permitAll()
	        	.antMatchers("/api/auth/**").permitAll()
	        	.antMatchers("/api/**").authenticated()
	            .anyRequest().permitAll()
	            .and()
	        .addFilterBefore(authFilter(), UsernamePasswordAuthenticationFilter.class)
	        .logout()
	        	.clearAuthentication(true)
	            .permitAll();
        
        http.csrf().disable();
        http.exceptionHandling().authenticationEntryPoint(authenticationEntryPoint());
        http.headers().frameOptions().sameOrigin();
        
        http.sessionManagement().sessionCreationPolicy(SessionCreationPolicy.IF_REQUIRED);
    }

    
    /**
     * @param auth
     * @throws Exception
     * @see org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter#configure(org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder)
     */
    @Override
    protected void configure(AuthenticationManagerBuilder auth) throws Exception {
        auth.eraseCredentials(true);
    }    

    
    /**
     * Auth filter.
     *
     * @return the json authentication filter
     * @throws Exception the exception
     */
    protected JsonAuthenticationFilter authFilter() throws Exception {
        JsonAuthenticationFilter filter = new JsonAuthenticationFilter();
        filter.setFilterProcessesUrl("/api/auth/login");
        filter.setAuthenticationSuccessHandler(new RestAuthenticationSuccessHandler());
        filter.setAuthenticationManager(authenticationManagerBean());
        filter.setUsernameParameter("email");
        return filter;
    }
    
    
    /**
     * Override this method to provide custom {@link AuthenticationEntryPoint}. 
     * 
     * It returns {@link RestAuthenticationEntryPoint} by default
     *
     * @return the authentication entry point
     */
    protected AuthenticationEntryPoint authenticationEntryPoint() {
    	return new RestAuthenticationEntryPoint();
    }

    
    
    /* *** LOGIN & SIGNUP configuration *** */
    
    /**
     * Auth client resources.
     *
     * @return the web mvc configurer adapter
     */
    @Bean
    public WebMvcConfigurerAdapter authClientResources() {
        return new WebMvcConfigurerAdapter() {

            @Override
            public void addResourceHandlers(final ResourceHandlerRegistry registry) {
                registry.addResourceHandler("/auth/**").addResourceLocations("classpath:/com/jh/common/security/controller/");
            }            
            
//            @Override
//            public void addViewControllers(ViewControllerRegistry registry) {
//                registry.addViewController("/auth/login").setViewName("forward:/auth/loginForm.html");
//                registry.addViewController("/auth/signup").setViewName("forward:/auth/signupForm.html");
//            }

        };
    }    

    
}
